@@ -56,8 +56,11 @@ st_area.sfc = function(x, ...) {
 		lwgeom::st_geod_area(x)
 	} else {
 		a = CPL_area(x) # ignores units: units of coordinates
-		if (! is.na(st_crs(x)))
+		if (! is.na(st_crs(x))) {
 			units(a) = crs_parameters(st_crs(x))$ud_unit^2 # coord units
+			if (!is.null(to_m <- st_crs(x)$to_meter))
+				a = a * to_m^2
+		}
 		a
 	}
 }
@@ -99,8 +102,11 @@ st_length = function(x) {
 		ret = CPL_length(x)
 		ret[is.nan(ret)] = NA
 		crs = st_crs(x)
-		if (! is.na(crs))
+		if (! is.na(crs)) {
 			units(ret) = crs_parameters(crs)$ud_unit
+			if (!is.null(to_m <- st_crs(x)$to_meter))
+				ret = ret * to_m
+		}
 		ret
 	}
 }
@@ -160,8 +166,8 @@ st_geos_binop = function(op, x, y, par = 0.0, pattern = NA_character_,
 #' @param ... ignored
 #' @param dist_fun deprecated
 #' @param by_element logical; if \code{TRUE}, return a vector with distance between the first elements of \code{x} and \code{y}, the second, etc. if \code{FALSE}, return the dense matrix with all pairwise distances.
-#' @param which character; for Cartesian coordinates only: one of \code{Euclidian}, \code{Haussdorf} or \code{Frechet}; for geodetic coordinates, great circle distances are computed; see details
-#' @param par for \code{which} equal to \code{Haussdorf} or \code{Frechet}, optionally use a value between 0 and 1 to densify the geometry
+#' @param which character; for Cartesian coordinates only: one of \code{Euclidian}, \code{Haussdorff} or \code{Frechet}; for geodetic coordinates, great circle distances are computed; see details
+#' @param par for \code{which} equal to \code{Haussdorff} or \code{Frechet}, optionally use a value between 0 and 1 to densify the geometry
 #' @param tolerance ignored if \code{st_is_longlat(x)} is \code{FALSE}; otherwise, if set to a positive value, the first distance smaller than \code{tolerance} will be returned, and true distance may be smaller; this may speed up computation. In meters, or a \code{units} object convertible to meters.
 #' @return If \code{by_element} is \code{FALSE} \code{st_distance} returns a dense numeric matrix of dimension length(x) by length(y); otherwise it returns a numeric vector of length \code{x} or \code{y}, the shorter one being recycled. Distances involving empty geometries are \code{NA}.
 #' @details great circle distance calculations use function \code{geod_inverse} from PROJ; see Karney, Charles FF, 2013, Algorithms for geodesics, Journal of Geodesy 87(1), 43--55
@@ -1179,101 +1185,6 @@ st_line_sample = function(x, n, density, type = "regular", sample = NULL) {
 	st_sfc(CPL_gdal_linestring_sample(x, distList), crs = st_crs(x))
 }
 
-#' Create a regular tesselation over the bounding box of an sf or sfc object
-#'
-#' Create a square or hexagonal grid over the bounding box of an sf or sfc object
-#' @param x object of class \link{sf} or \link{sfc}
-#' @param cellsize target cellsize
-#' @param offset numeric of lengt 2; lower left corner coordinates (x, y) of the grid
-#' @param n integer of length 1 or 2, number of grid cells in x and y direction (columns, rows)
-#' @param crs object of class \code{crs}; coordinate reference system of the target of the target grid in case argument \code{x} is missing, if \code{x} is not missing, its crs is inherited.
-#' @param what character; one of: \code{"polygons"}, \code{"corners"}, or \code{"centers"}
-#' @param square logical; if \code{FALSE}, create hexagonal grid
-#' @return Object of class \code{sfc} (simple feature geometry list column) with, depending on \code{what} and \code{square},
-#' square or hexagonal polygons, corner points of these polygons, or center points of these polygons.
-#' @examples
-#' plot(st_make_grid(what = "centers"), axes = TRUE)
-#' plot(st_make_grid(what = "corners"), add = TRUE, col = 'green', pch=3)
-#' sfc = st_sfc(st_polygon(list(rbind(c(0,0), c(1,0), c(1,1), c(0,0)))))
-#' plot(st_make_grid(sfc, cellsize = .1, square = FALSE))
-#' plot(sfc, add = TRUE)
-#' # non-default offset:
-#' plot(st_make_grid(sfc, cellsize = .1, square = FALSE, offset = c(0, .05 / (sqrt(3)/2))))
-#' plot(sfc, add = TRUE)
-#' @export
-st_make_grid = function(x,
-		cellsize = c(diff(st_bbox(x)[c(1,3)]), diff(st_bbox(x)[c(2,4)]))/n,
-		offset = st_bbox(x)[c("xmin", "ymin")], n = c(10, 10),
-		crs = if (missing(x)) NA_crs_ else st_crs(x),
-		what = "polygons", square = TRUE) {
-
-	if (missing(x) && missing(cellsize) && missing(offset)
-			&& missing(n) && missing(crs)) # create global 10 x 10 degree grid
-		return(st_make_grid(cellsize = c(10,10), offset = c(-180,-90), n = c(36,18),
-			crs = st_crs(4326), what = what))
-
-	if (! square)
-		return(hex_grid(x, dx = cellsize[1], pt = offset, points = what != "polygons", clip = TRUE))
-
-	bb = if (!missing(n) && !missing(offset) && !missing(cellsize)) {
-		cellsize = rep(cellsize, length.out = 2)
-		n = rep(n, length.out = 2)
-		bb_wrap(c(offset, offset + n * cellsize))
-	} else
-		st_bbox(x)
-
-	cellsize_missing = if (! missing(cellsize)) {
-		cellsize = rep(cellsize, length.out = 2)
-		FALSE
-	} else
-		TRUE
-
-	if (missing(n)) {
-		nx = ceiling((bb[3] - offset[1])/cellsize[1])
-		ny = ceiling((bb[4] - offset[2])/cellsize[2])
-	} else {
-		n = rep(n, length.out = 2)
-		nx = n[1]
-		ny = n[2]
-	}
-
-	# corner points:
-	if (cellsize_missing) {
-		xc = seq(offset[1], bb[3], length.out = nx + 1)
-		yc = seq(offset[2], bb[4], length.out = ny + 1)
-	} else {
-		xc = offset[1] + (0:nx) * cellsize[1]
-		yc = offset[2] + (0:ny) * cellsize[2]
-	}
-
-	if (what == "polygons") {
-		ret = vector("list", nx * ny)
-		square = function(x1, y1, x2, y2)
-			st_polygon(list(matrix(c(x1, x2, x2, x1, x1, y1, y1, y2, y2, y1), 5)))
-		for (i in 1:nx)
-			for (j in 1:ny)
-				ret[[(j - 1) * nx + i]] = square(xc[i], yc[j], xc[i+1], yc[j+1])
-	} else if (what == "centers") {
-		ret = vector("list", nx * ny)
-		cent = function(x1, y1, x2, y2)
-			st_point(c( (x1+x2)/2, (y1+y2)/2 ))
-		for (i in 1:nx)
-			for (j in 1:ny)
-				ret[[(j - 1) * nx + i]] = cent(xc[i], yc[j], xc[i+1], yc[j+1])
-	} else if (what == "corners") {
-		ret = vector("list", (nx + 1) * (ny + 1))
-		for (i in 1:(nx + 1))
-			for (j in 1:(ny + 1))
-				ret[[(j - 1) * (nx + 1) + i]] = st_point(c(xc[i], yc[j]))
-	} else
-		stop("unknown value of `what'")
-
-	if (missing(x))
-		st_sfc(ret, crs = crs)
-	else
-		st_sfc(ret, crs = st_crs(x))
-}
-
 #' Internal functions
 #' @name internal
 #' @param msg error message
