@@ -28,7 +28,9 @@ set_utf8 = function(x) {
 #'   use the \code{query} argument.
 #' @param ... parameter(s) passed on to \link{st_as_sf}
 #' @param options character; driver dependent dataset open options, multiple
-#'   options supported.
+#'   options supported. For possible values, see the "Open options" section
+#'   of the GDAL documentation of the corresponding driver, and 
+#'   https://github.com/r-spatial/sf/issues/1157 for an example.
 #' @param quiet logical; suppress info on name, driver, size and spatial
 #'   reference, or signaling no or multiple layers
 #' @param geometry_column integer or character; in case of multiple geometry
@@ -42,8 +44,9 @@ set_utf8 = function(x) {
 #'   of LineString and MultiLineString, or of Polygon and MultiPolygon, convert
 #'   all to the Multi variety; defaults to \code{TRUE}
 #' @param stringsAsFactors logical; logical: should character vectors be
-#'   converted to factors?  The `factory-fresh' default is \code{TRUE}, but this
-#'   can be changed by setting \code{options(stringsAsFactors = FALSE)}.
+#'   converted to factors?  The `factory-fresh' default is \code{TRUE} for 
+#'   \code{st_read} and \code{FALSE} for \code{read_sf}, but this can be changed 
+#'   globally by e.g. the R command \code{options(stringsAsFactors = FALSE)}.
 #' @param int64_as_string logical; if TRUE, Int64 attributes are returned as
 #'   string; if FALSE, they are returned as double and a warning is given when
 #'   precision is lost (i.e., values are larger than 2^53).
@@ -108,6 +111,10 @@ set_utf8 = function(x) {
 #' nc_gpkg_sql = st_read(system.file("gpkg/nc.gpkg", package = "sf"),
 #'    query = sprintf("SELECT NAME, SID74, FIPS, geom  FROM \"%s\" WHERE BIR74 > 20000", layer))
 #' }
+#' # spatial filter, as wkt:
+#' wkt = st_as_text(st_geometry(nc[1,])) 
+#' # filter by (bbox overlaps of) first feature geometry:
+#' read_sf(system.file("gpkg/nc.gpkg", package="sf"), wkt_filter = wkt)
 #' @export
 st_read = function(dsn, layer, ...) UseMethod("st_read")
 
@@ -164,7 +171,7 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 		x <- as.data.frame(set_utf8(x[-c(lc.other, which.geom)]), stringsAsFactors = stringsAsFactors)
 		if (as_tibble) {
 			# "sf" class is added later by `st_as_sf` (and sets all the attributes)
-			x <- tibble::new_tibble(x, attributes(x), nrow = nrow(x))
+			x <- tibble::new_tibble(x, nrow = nrow(x))
 		}
 	}
 
@@ -185,6 +192,8 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 
 #' @name st_read
 #' @param fid_column_name character; name of column to write feature IDs to; defaults to not doing this
+#' @param drivers character; limited set of driver short names to be tried (default: try all)
+#' @param wkt_filter character; WKT representation of a spatial filter (may be used as bounding box, selecting overlapping geometries); see examples
 #' @note The use of \code{system.file} in examples make sure that examples run regardless where R is installed:
 #' typical users will not use \code{system.file} but give the file name directly, either with full path or relative
 #' to the current working directory (see \link{getwd}). "Shapefiles" consist of several files with the same basename
@@ -192,7 +201,8 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 #' @export
 st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet = FALSE, geometry_column = 1L, type = 0,
 		promote_to_multi = TRUE, stringsAsFactors = default.stringsAsFactors(),
-		int64_as_string = FALSE, check_ring_dir = FALSE, fid_column_name = character(0)) {
+		int64_as_string = FALSE, check_ring_dir = FALSE, fid_column_name = character(0),
+		drivers = character(0), wkt_filter = character(0)) {
 
 	layer = if (missing(layer))
 		character(0)
@@ -210,7 +220,7 @@ st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet
 		stop("`promote_to_multi' should have length one, and applies to all geometry columns")
 
 	x = CPL_read_ogr(dsn, layer, query, as.character(options), quiet, type, fid_column_name,
-		promote_to_multi, int64_as_string, dsn_exists, dsn_isdb)
+		drivers, wkt_filter, promote_to_multi, int64_as_string, dsn_exists, dsn_isdb)
 	process_cpl_read_ogr(x, quiet, check_ring_dir = check_ring_dir,
 		stringsAsFactors = stringsAsFactors, geometry_column = geometry_column, ...)
 }
@@ -253,6 +263,10 @@ clean_columns = function(obj, factorsAsCharacter) {
 	ccls.ok = vapply(obj, function(x) inherits(x, permitted), TRUE)
 	if (any(!ccls.ok)) {
 		# nocov start
+                nms <- names(obj)[!ccls.ok]
+                cls <- sapply(obj, function(x) paste(class(x), collapse=";"))[!ccls.ok]
+                warning("Dropping column(s) ", paste(nms, collapse=","),
+                    " of class(es) ", paste(cls, collapse=","))
 		obj = obj[ccls.ok]
 		# nocov end
 	}
@@ -297,23 +311,39 @@ abbreviate_shapefile_names = function(x) {
 #' are found at \url{http://www.gdal.org/ogr_formats.html}.
 #' @param ... other arguments passed to \link{dbWriteTable} when \code{dsn} is a
 #' Database Connection
-#' @param dataset_options character; driver dependent dataset creation options; multiple options supported.
-#' @param layer_options character; driver dependent layer creation options; multiple options supported.
+#' @param dataset_options character; driver dependent dataset creation options; 
+#' multiple options supported.
+#' @param layer_options character; driver dependent layer creation options; 
+#' multiple options supported.
 #' @param quiet logical; suppress info on name, driver, size and spatial reference
-#' @param factorsAsCharacter logical; convert \code{factor} objects into character strings (default), else into numbers by
-#' \code{as.numeric}.
-#' @param update logical; \code{FALSE} by default for single-layer drivers but \code{TRUE} by default for database drivers
-#' as defined by \code{db_drivers}.
-#' For database-type drivers (e.g. GPKG) \code{TRUE} values will make \code{GDAL} try
-#' to update (append to) the existing data source, e.g. adding a table to an existing database.
-#' @param delete_dsn logical; delete data source \code{dsn} before attempting to write?
-#' @param delete_layer logical; delete layer \code{layer} before attempting to write? (not yet implemented)
-#' @param fid_column_name character, name of column with feature IDs; if specified, this column is no longer written as feature attribute.
-#' @details columns (variables) of a class not supported are dropped with a warning. When deleting layers or
-#' data sources is not successful, no error is emitted. \code{delete_dsn} and \code{delete_layers} should be
+#' @param factorsAsCharacter logical; convert \code{factor} objects into 
+#' character strings (default), else into numbers by \code{as.numeric}.
+#' @param update logical; \code{FALSE} by default for single-layer drivers but 
+#' \code{TRUE} by default for database drivers as defined by \code{db_drivers}.
+#' For database-type drivers (e.g. GPKG) \code{TRUE} values will make 
+#' \code{GDAL} try to update (append to) the existing data source, e.g. adding
+#' a table to an existing database, or adding records to a layer. See also the
+#' next two arguments and Details.
+#' @param delete_dsn logical; delete data source \code{dsn} before attempting 
+#' to write?
+#' @param delete_layer logical; delete layer \code{layer} before attempting to
+#' write?
+#' @param fid_column_name character, name of column with feature IDs; if
+#' specified, this column is no longer written as feature attribute.
+#' @details 
+#' Columns (variables) of a class not supported are dropped with a warning. 
+#' 
+#' When updating an existing layer, records are appended to it if the updating
+#' object has the right variable names and types. If names don't match an 
+#' error is raised. If types don't match, behaviour is undefined: GDAL may
+#' raise warnings or errors or fail silently.
+#' 
+#' When deleting layers or data sources is not successful, no error is emitted. 
+#' \code{delete_dsn} and \code{delete_layer} should be
 #' handled with care; the former may erase complete directories or databases.
 #' @seealso \link{st_drivers}
-#' @return \code{obj}, invisibly; in case \code{obj} is of class \code{sfc}, it is returned as an  \code{sf} object.
+#' @return \code{obj}, invisibly; in case \code{obj} is of class \code{sfc}, 
+#' it is returned as an \code{sf} object.
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
 #' st_write(nc, paste0(tempdir(), "/", "nc.shp"))
@@ -371,7 +401,7 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 	}
 
 	if (length(list(...)))
-		stop(paste("unrecognized argument(s)", unlist(list(...)), "\n"))
+		stop(paste("unrecognized argument(s)", names(list(...)), "\n"))
 	if (is.null(layer))
 		layer <- file_path_sans_ext(basename(dsn))
 
