@@ -5,7 +5,7 @@ WKT_name = function(x, EWKT = TRUE) {
 
 	retval = if (zm == "")
 		cls[2]
-	else 
+	else
 		paste(cls[2], substr(cls[1], 3, 4))
 
 	if (EWKT && !is.null(attr(x, "epsg")) && !is.na(attr(x, "epsg")))
@@ -23,15 +23,17 @@ fmt = function(x, ...) sub("^[ ]+", "", sapply(unclass(x), format, ...))
 prnt.POINT = function(x, ..., EWKT = TRUE) {
 	pt = if (any(!is.finite(x)))
 		empty
-	else 
+	else
 		paste0("(", paste0(fmt(x, ...), collapse = " "), ")")
 	paste(WKT_name(x, EWKT = EWKT), pt)
 }
 
-prnt.Matrix = function(x, ...) {
+prnt.Matrix = function(x, nested_parens = FALSE, ...) {
 	pf = function(x, ..., collapse) paste0(fmt(x, ...), collapse = collapse)
 	if (nrow(x) == 0)
 		empty
+	else if (nested_parens)
+		paste0("((", paste0(apply(x, 1, pf, collapse = " ", ...), collapse = "), ("), "))")
 	else
 		paste0("(", paste0(apply(x, 1, pf, collapse = " ", ...), collapse = ", "), ")")
 }
@@ -50,7 +52,10 @@ prnt.MatrixListList = function(x, ...) {
 		paste0("(", paste0(unlist(lapply(x, prnt.MatrixList, ...)), collapse = ", "), ")")
 }
 
-prnt.MULTIPOINT = function(x, ..., EWKT = TRUE) paste(WKT_name(x, EWKT = EWKT), prnt.Matrix(x, ...))
+prnt.MULTIPOINT = function(x, ..., EWKT = TRUE, nested_parens = FALSE) {
+	paste(WKT_name(x, EWKT = EWKT),
+		  prnt.Matrix(x, nested_parens = nested_parens, ...))
+}
 prnt.LINESTRING = function(x, ..., EWKT = TRUE) paste(WKT_name(x, EWKT = EWKT), prnt.Matrix(x, ...))
 prnt.POLYGON = function(x, ..., EWKT = TRUE) paste(WKT_name(x, EWKT = EWKT), prnt.MatrixList(x, ...))
 prnt.MULTILINESTRING = function(x, ..., EWKT = TRUE) paste(WKT_name(x, EWKT = EWKT), prnt.MatrixList(x, ...))
@@ -83,9 +88,12 @@ st_as_text = function(x, ...) UseMethod("st_as_text")
 #' st_as_text(st_point(1:2))
 #' st_as_text(st_sfc(st_point(c(-90,40)), crs = 4326), EWKT = TRUE)
 st_as_text.sfg = function(x, ...) {
-	switch(class(x)[2],
+	if (Sys.getenv("LWGEOM_WKT") == "true" && requireNamespace("lwgeom", quietly = TRUE) && utils::packageVersion("lwgeom") >= "0.1-5")
+		lwgeom::st_astext(x, ...)
+	else
+	  switch(class(x)[2],
 		POINT = prnt.POINT(x, ...),
-		MULTIPOINT =        prnt.MULTIPOINT(x, ...),
+		MULTIPOINT =        prnt.MULTIPOINT(x, ..., nested_parens = TRUE),
 		LINESTRING =        prnt.LINESTRING(x, ...),
 		POLYGON =           prnt.POLYGON(x, ...),
 		MULTILINESTRING =   prnt.MULTILINESTRING(x, ...),
@@ -108,12 +116,16 @@ st_as_text.sfg = function(x, ...) {
 #' @param EWKT logical; if TRUE, print SRID=xxx; before the WKT string if \code{epsg} is available
 #' @export
 st_as_text.sfc = function(x, ..., EWKT = FALSE) {
-	if (EWKT) {
-		epsg = attr(x, "crs")$epsg
-		if (!is.na(epsg) && epsg != 0)
-			x = lapply(x, function(sfg) structure(sfg, epsg = epsg))
+	if (Sys.getenv("LWGEOM_WKT") == "true" && requireNamespace("lwgeom", quietly = TRUE) && utils::packageVersion("lwgeom") >= "0.1-5")
+		lwgeom::st_astext(x, ..., EWKT = EWKT)
+	else {
+		if (EWKT) {
+			epsg = attr(x, "crs")$epsg
+			if (!is.na(epsg) && epsg != 0)
+				x = lapply(x, function(sfg) structure(sfg, epsg = epsg))
+		}
+		vapply(x, st_as_text, "", ..., EWKT = EWKT)
 	}
-	vapply(x, st_as_text, "", ..., EWKT = EWKT)
 }
 
 #' @name st_as_sfc
