@@ -14,9 +14,9 @@ else
 	system.file("shape/nc.shp", package="sf")
 }
 
-rtg = c(AREA = "lattice", PERIMETER = "lattice", CNTY_ = "entity",
-		CNTY_ID = "entity", NAME = "entity", FIPS = "entity", FIPSNO = "entity",
-		CRESS_ID = "entity", BIR74 = "lattice", SID74 = "lattice", NWBIR74 = "lattice",
-		BIR79 = "lattice", SID79 = "lattice", NWBIR79  = "lattice")
+agr = c(AREA = "aggregate", PERIMETER = "aggregate", CNTY_ = "identity",
+		CNTY_ID = "identity", NAME = "identity", FIPS = "identity", FIPSNO = "identity",
+		CRESS_ID = "identity", BIR74 = "aggregate", SID74 = "aggregate", NWBIR74 = "aggregate",
+		BIR79 = "aggregate", SID79 = "aggregate", NWBIR79  = "aggregate")
 
-nc = st_read(datasource, relation_to_geometry = rtg)
+nc = st_read(datasource, agr = agr)
