@@ -201,7 +201,7 @@ std::vector<char *> create_options(Rcpp::CharacterVector lco, bool quiet) {
 	if (lco.size() == 0)
 		quiet = true; // nothing to report
 	if (! quiet)
-		Rcpp::Rcout << "options:        "; // #nocov
+		Rcpp::Rcout <<  "options:        "; // #nocov
 	std::vector<char *> ret(lco.size() + 1);
 	for (int i = 0; i < lco.size(); i++) {
 		ret[i] = (char *) (lco[i]);
@@ -266,11 +266,14 @@ Rcpp::List get_crs(OGRSpatialReference *ref) {
 }
 
 Rcpp::List sfc_from_ogr(std::vector<OGRGeometry *> g, bool destroy = false) {
+	OGRwkbGeometryType type = wkbGeometryCollection;
 	Rcpp::List lst(g.size());
 	Rcpp::List crs = get_crs(g.size() && g[0] != NULL ? g[0]->getSpatialReference() : NULL);
 	for (size_t i = 0; i < g.size(); i++) {
 		if (g[i] == NULL)
-			Rcpp::stop("NULL error in sfc_from_ogr"); // #nocov
+			g[i] = OGRGeometryFactory::createGeometry(type); // #nocov
+		else
+			type = g[i]->getGeometryType();
 		Rcpp::RawVector raw(g[i]->WkbSize());
 		handle_error(g[i]->exportToWkb(wkbNDR, &(raw[0]), wkbVariantIso));
 		lst[i] = raw;
@@ -387,8 +390,9 @@ Rcpp::List CPL_transform(Rcpp::List sfc, Rcpp::CharacterVector proj4) {
 	OGRCoordinateTransformation *ct = 
 		OGRCreateCoordinateTransformation(g[0]->getSpatialReference(), dest);
 	if (ct == NULL) {
-		dest->Release(); // #nocov
-		Rcpp::stop("OGRCreateCoordinateTransformation() returned NULL: PROJ available?"); // #nocov
+		dest->Release(); // #nocov start
+		sfc_from_ogr(g, true); // to destroy g
+		Rcpp::stop("OGRCreateCoordinateTransformation() returned NULL: PROJ available?"); // #nocov end
 	}
 	for (size_t i = 0; i < g.size(); i++) {
 		CPLPushErrorHandler(CPLQuietErrorHandler);
@@ -398,9 +402,8 @@ Rcpp::List CPL_transform(Rcpp::List sfc, Rcpp::CharacterVector proj4) {
 		CPLPopErrorHandler();
 		if (err == 1 || err == 6) {
 			OGRwkbGeometryType geomType = g[i]->getGeometryType();
-			OGRGeometryFactory f;
-			f.destroyGeometry(g[i]);
-			g[i] = f.createGeometry(geomType);
+			OGRGeometryFactory::destroyGeometry(g[i]);
+			g[i] = OGRGeometryFactory::createGeometry(geomType); // return empty geometry of this type
 		} else
 			handle_error(err);
 	}
