@@ -479,16 +479,18 @@ Rcpp::List sf_from_ogrlayer(OGRLayer *poLayer, bool quiet, bool int64_as_string,
 Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector layer,
 		Rcpp::CharacterVector query,
 		Rcpp::CharacterVector options, bool quiet, Rcpp::NumericVector toTypeUser,
-		Rcpp::CharacterVector fid_column_name,
+		Rcpp::CharacterVector fid_column_name, Rcpp::CharacterVector drivers,
+		Rcpp::CharacterVector wkt_filter,
 		bool promote_to_multi = true, bool int64_as_string = false,
 		bool dsn_exists = true,
 		bool dsn_isdb = false) {
 
 	// adapted from the OGR tutorial @ www.gdal.org
 	std::vector <char *> open_options = create_options(options, quiet);
+	std::vector <char *> drivers_v = create_options(drivers, quiet);
 	GDALDataset *poDS;
-	poDS = (GDALDataset *) GDALOpenEx( datasource[0], GDAL_OF_VECTOR | GDAL_OF_READONLY, NULL,
-		open_options.data(), NULL );
+	poDS = (GDALDataset *) GDALOpenEx( datasource[0], GDAL_OF_VECTOR | GDAL_OF_READONLY, 
+		drivers.size() ? drivers_v.data() : NULL, open_options.data(), NULL );
 	if( poDS == NULL ) {
 		// could not open dsn
 		if( dsn_isdb ) {
@@ -540,6 +542,23 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 		Rcpp::stop("Opening layer failed.\n");
 	}
 
+	// set spatial filter?
+	if (wkt_filter.size()) {
+		char *wkt = wkt_filter[0];
+		OGRGeometry *new_geom;
+#if GDAL_VERSION_MAJOR <= 2 && GDAL_VERSION_MINOR <= 2
+		OGRErr err = OGRGeometryFactory::createFromWkt(&wkt, poLayer->GetSpatialRef(), &new_geom);
+#else
+		OGRErr err = OGRGeometryFactory::createFromWkt((const char *) wkt, poLayer->GetSpatialRef(), &new_geom);
+#endif
+		if (err != OGRERR_NONE) {
+			Rcpp::Rcout << "Cannot create geometry from: " << wkt_filter[0] << std::endl;
+			Rcpp::stop("wkt parse error.\n");
+		}
+		poLayer->SetSpatialFilter(new_geom);
+		OGRGeometryFactory::destroyGeometry(new_geom);
+	}
+
 	if (! quiet)
 		Rcpp::Rcout << "Reading layer `" << layer[0] << "' from data source `" << datasource[0] << // #nocov
 			"' using driver `" << poDS->GetDriverName() << "'" << std::endl;                       // #nocov
