@@ -3,23 +3,32 @@
 #include "gdal_version.h"
 #include "gdalwarper.h"
 
-#if (!(GDAL_VERSION_MAJOR == 2 && GDAL_VERSION_MINOR < 1))
-# include "gdal_utils.h" // requires >= 2.1
+#include <ogr_srs_api.h>
+#include <ogr_spatialref.h>
+#include "gdal.h" // local
 
 #include "Rcpp.h"
 
 #define NO_GDAL_CPP_HEADERS
 #include "gdal_sf_pkg.h"
 
+#if (!(GDAL_VERSION_MAJOR == 2 && GDAL_VERSION_MINOR < 1))
+# include "gdal_utils.h" // requires >= 2.1
+
 // [[Rcpp::export]]
-Rcpp::CharacterVector CPL_gdalinfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options) {
+Rcpp::CharacterVector CPL_gdalinfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options, 
+		Rcpp::CharacterVector oo) {
 	std::vector <char *> options_char = create_options(options, true);
+	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALInfoOptions* opt = GDALInfoOptionsNew(options_char.data(), NULL);
-	GDALDatasetH ds = GDALOpen((const char *) obj[0], GA_ReadOnly);
+	GDALDatasetH ds = GDALOpenEx((const char *) obj[0], GA_ReadOnly, NULL, oo_char.data(), NULL);
+	if (ds == NULL)
+		return 1; // #nocov
 	char *ret_val = GDALInfo(ds, opt);
 	Rcpp::CharacterVector ret = ret_val; // copies
 	CPLFree(ret_val);
 	GDALInfoOptionsFree(opt);
+	GDALClose(ds);
 	return ret;
 }
 
@@ -27,18 +36,23 @@ Rcpp::CharacterVector CPL_gdalinfo(Rcpp::CharacterVector obj, Rcpp::CharacterVec
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalwarp(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
 
 	int err = 0;
 
+	std::vector <char *> oo_char = create_options(oo, true); // open options
 	std::vector<GDALDatasetH> src_pt(src.size());
 	for (int i = 0; i < src.size(); i++)
-		src_pt[i] = GDALOpen((const char *) src[i], GA_ReadOnly);
+		src_pt[i] = GDALOpenEx((const char *) src[i], GA_ReadOnly, NULL, oo_char.data(), NULL);
+
+	std::vector <char *> doo_char = create_options(doo, true); // open options
+	GDALDatasetH dst_ds = GDALOpenEx((const char *) dst[0], GDAL_OF_RASTER | GA_Update, NULL, doo_char.data(), NULL);
 
 	std::vector <char *> options_char = create_options(options, true);
 	GDALWarpAppOptions* opt = GDALWarpAppOptionsNew(options_char.data(), NULL);
 
-	GDALDatasetH result = GDALWarp((const char *) dst[0], NULL, src.size(), src_pt.data(), opt, &err);
+	GDALDatasetH result = GDALWarp(dst_ds == NULL ? (const char *) dst[0] : NULL, dst_ds, 
+		src.size(), src_pt.data(), opt, &err);
 	GDALWarpAppOptionsFree(opt);
 	for (int i = 0; i < src.size(); i++)
 		if (src_pt[i] != NULL)
@@ -50,21 +64,30 @@ Rcpp::LogicalVector CPL_gdalwarp(Rcpp::CharacterVector src, Rcpp::CharacterVecto
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalrasterize(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo,
+		bool overwrite = false) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
+	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALRasterizeOptions* opt =  GDALRasterizeOptionsNew(options_char.data(), NULL);
 
-	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_VECTOR | GA_ReadOnly, NULL, NULL, NULL);
+	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_VECTOR | GA_ReadOnly, 
+		NULL, oo_char.data(), NULL);
 	if (src_pt == NULL)
 		Rcpp::stop("source dataset not found");
-	GDALDatasetH dst_pt = GDALOpen((const char *) dst[0], GA_Update);
-	if (dst_pt == NULL)
-		Rcpp::stop("cannot write to destination dataset");
-	GDALDatasetH result = GDALRasterize(NULL, dst_pt, src_pt, opt, &err);
+	unset_error_handler();
+	GDALDatasetH dst_pt = NULL;
+	if (! overwrite) {
+		std::vector <char *> doo_char = create_options(doo, true); // open options
+		dst_pt = GDALOpenEx((const char *) dst[0], GDAL_OF_RASTER | GA_Update, NULL, doo_char.data(), NULL);
+	}
+	set_error_handler();
+	GDALDatasetH result = 
+		GDALRasterize(dst_pt == NULL ? (const char *) dst[0] : NULL, dst_pt, src_pt, opt, &err);
 	GDALRasterizeOptionsFree(opt);
-	GDALClose(src_pt);
+	if (src_pt != NULL)
+		GDALClose(src_pt);
 	if (result != NULL)
 		GDALClose(result);
 	return result == NULL || err;
@@ -74,14 +97,20 @@ Rcpp::LogicalVector CPL_gdalrasterize(Rcpp::CharacterVector src, Rcpp::Character
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdaltranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
+	std::vector <char *> oo_char = create_options(oo, true);
 	GDALTranslateOptions* opt =  GDALTranslateOptionsNew(options_char.data(), NULL);
 
-	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, NULL, NULL, NULL);
+	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, 
+		NULL, oo_char.data(), NULL);
+	if (src_pt == NULL)
+		return 1; // #nocov
 	GDALDatasetH result = GDALTranslate((const char *) dst[0], src_pt, opt, &err);
+	if (src_pt != NULL)
+		GDALClose(src_pt);
 	GDALTranslateOptionsFree(opt);
 	if (result != NULL)
 		GDALClose(result);
@@ -90,16 +119,25 @@ Rcpp::LogicalVector CPL_gdaltranslate(Rcpp::CharacterVector src, Rcpp::Character
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalvectortranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
 	GDALVectorTranslateOptions* opt =  GDALVectorTranslateOptionsNew(options_char.data(), NULL);
 
-	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_VECTOR | GA_ReadOnly, NULL, NULL, NULL);
-	// GDALDatasetH dst_pt = GDALOpen((const char *) dst[0], GA_Update);
-	GDALDatasetH result = GDALVectorTranslate((const char *) dst[0], NULL, 1, &src_pt, opt, &err);
+	std::vector <char *> oo_char = create_options(oo, true); // open options
+	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_VECTOR | GA_ReadOnly, NULL, 
+		oo_char.data(), NULL);
+	if (src_pt == NULL)
+		return 1; // #nocov
+	std::vector <char *> doo_char = create_options(doo, true); // open options
+	unset_error_handler();
+	GDALDatasetH dst_pt = GDALOpenEx((const char *) dst[0], GDAL_OF_VECTOR | GA_Update, NULL, doo_char.data(), NULL);
+	set_error_handler();
+	GDALDatasetH result = 
+		GDALVectorTranslate(dst_pt == NULL ? (const char *) dst[0] : NULL, dst_pt, 1, &src_pt, opt, &err);
 	GDALVectorTranslateOptionsFree(opt);
+	GDALClose(src_pt);
 	if (result != NULL)
 		GDALClose(result);
 	return result == NULL || err;
@@ -107,19 +145,28 @@ Rcpp::LogicalVector CPL_gdalvectortranslate(Rcpp::CharacterVector src, Rcpp::Cha
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalbuildvrt(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
+	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALBuildVRTOptions* opt = GDALBuildVRTOptionsNew(options_char.data(), NULL);
 
+	/*
 	std::vector<const char *> srcpt(src.size());
 	for (int i = 0; i < src.size(); i++)
 		srcpt[i] = (const char *) src[i];
+	*/
+	std::vector<GDALDatasetH> srcpt(src.size());
+	for (int i = 0; i < src.size(); i++)
+		srcpt[i] = GDALOpenEx((const char *) src[i], GDAL_OF_RASTER | GA_ReadOnly, NULL, 
+			oo_char.data(), NULL);
 
-	GDALDatasetH result = GDALBuildVRT((const char *) dst[0], src.size(), NULL, srcpt.data(), opt, &err);
+	GDALDatasetH result = GDALBuildVRT((const char *) dst[0], src.size(), srcpt.data(), NULL, opt, &err);
 
 	GDALBuildVRTOptionsFree(opt);
+	for (int i = 0; i < src.size(); i++)
+		GDALClose(srcpt[i]);
 	if (result != NULL)
 		GDALClose(result);
 	return result == NULL || err;
@@ -127,13 +174,16 @@ Rcpp::LogicalVector CPL_gdalbuildvrt(Rcpp::CharacterVector src, Rcpp::CharacterV
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdaldemprocessing(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector processing, Rcpp::CharacterVector colorfilename) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector processing, Rcpp::CharacterVector colorfilename,
+		Rcpp::CharacterVector oo) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
+	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALDEMProcessingOptions* opt =  GDALDEMProcessingOptionsNew(options_char.data(), NULL);
 
-	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, NULL, NULL, NULL);
+	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, 
+		NULL, oo_char.data(), NULL);
 	if (src_pt == NULL)
 		Rcpp::stop("cannot open source dataset"); // #nocov
 	GDALDatasetH result = GDALDEMProcessing((const char *) dst[0], src_pt, 
@@ -143,23 +193,28 @@ Rcpp::LogicalVector CPL_gdaldemprocessing(Rcpp::CharacterVector src, Rcpp::Chara
 	GDALDEMProcessingOptionsFree(opt);
 	if (result != NULL)
 		GDALClose(result);
+	if (src_pt != NULL)
+		GDALClose(src_pt);
 	return result == NULL || err;
 }
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalnearblack(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
+	std::vector <char *> oo_char = create_options(oo, true); // open options
+	std::vector <char *> doo_char = create_options(doo, true); // open options
 	GDALNearblackOptions* opt =  GDALNearblackOptionsNew(options_char.data(), NULL);
 
 	// GDALDatasetH src_pt = GDALOpen((const char *) src[0], GA_ReadOnly);
-	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, NULL, NULL, NULL);
-	GDALDatasetH dst_pt = GDALOpen((const char *) dst[0], GA_Update);
-	GDALDatasetH result = GDALNearblack(NULL, dst_pt, src_pt, opt, &err);
+	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, NULL, oo_char.data(), NULL);
+	GDALDatasetH dst_pt = GDALOpenEx((const char *) dst[0], GDAL_OF_RASTER | GA_Update, NULL, doo_char.data(), NULL);
+	GDALDatasetH result = GDALNearblack(dst_pt == NULL ? (const char *) dst[0] : NULL, dst_pt, src_pt, opt, &err);
 	GDALNearblackOptionsFree(opt);
-	GDALClose(src_pt);
+	if (src_pt != NULL) 
+		GDALClose(src_pt);
 	if (result != NULL)
 		GDALClose(result);
 	return result == NULL || err;
@@ -167,16 +222,19 @@ Rcpp::LogicalVector CPL_gdalnearblack(Rcpp::CharacterVector src, Rcpp::Character
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalgrid(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
+	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALGridOptions* opt =  GDALGridOptionsNew(options_char.data(), NULL);
 
-	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_ALL | GA_ReadOnly, NULL, NULL, NULL);
+	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_ALL | GA_ReadOnly, 
+		NULL, oo_char.data(), NULL);
 	GDALDatasetH result = GDALGrid((const char *) dst[0], src_pt, opt, &err);
 	GDALGridOptionsFree(opt);
-	GDALClose(src_pt);
+	if (src_pt != NULL)
+		GDALClose(src_pt);
 	if (result != NULL)
 		GDALClose(result);
 	return result == NULL || err;
@@ -185,47 +243,50 @@ Rcpp::LogicalVector CPL_gdalgrid(Rcpp::CharacterVector src, Rcpp::CharacterVecto
 #else
 #include "Rcpp.h"
 
-Rcpp::CharacterVector CPL_gdalinfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options) {
+Rcpp::CharacterVector CPL_gdalinfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options, 
+		Rcpp::CharacterVector oo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 
 Rcpp::LogicalVector CPL_gdalwarp(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 
 Rcpp::LogicalVector CPL_gdalrasterize(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo,
+		bool overwrite = false) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 
 Rcpp::LogicalVector CPL_gdaltranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 
 Rcpp::LogicalVector CPL_gdalvectortranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 
 Rcpp::LogicalVector CPL_gdalbuildvrt(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 
 Rcpp::LogicalVector CPL_gdaldemprocessing(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector processing, Rcpp::CharacterVector colorfilename) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector processing, Rcpp::CharacterVector colorfilename,
+		Rcpp::CharacterVector oo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 
 Rcpp::LogicalVector CPL_gdalnearblack(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 
 Rcpp::LogicalVector CPL_gdalgrid(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
 #endif
@@ -234,15 +295,17 @@ Rcpp::LogicalVector CPL_gdalgrid(Rcpp::CharacterVector src, Rcpp::CharacterVecto
 // https://www.gdal.org/warptut.html :
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdal_warper(Rcpp::CharacterVector infile, Rcpp::CharacterVector outfile,
-		Rcpp::IntegerVector options)
-{
+		Rcpp::IntegerVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
+
+	std::vector <char *> oo_char = create_options(oo, true); // open options
     GDALDatasetH  hSrcDS, hDstDS;
     // Open input and output files.
     GDALAllRegister();
-    hSrcDS = GDALOpen( infile[0], GA_ReadOnly );
+    hSrcDS = GDALOpenEx( infile[0], GA_ReadOnly, NULL, oo_char.data(), NULL );
 	if (hSrcDS == NULL)
 		Rcpp::stop("input file not found");
-    hDstDS = GDALOpen( outfile[0], GA_Update );
+	std::vector <char *> doo_char = create_options(doo, true); // open options
+    hDstDS = GDALOpenEx(outfile[0], GA_Update, NULL, doo_char.data(), NULL);
 	if (hDstDS == NULL)
 		Rcpp::stop("could not open output file for writing");
     // Setup warp options.
@@ -298,8 +361,10 @@ Rcpp::LogicalVector CPL_gdal_warper(Rcpp::CharacterVector infile, Rcpp::Characte
                                   GDALGetRasterYSize( hDstDS ) );
     GDALDestroyGenImgProjTransformer( psWarpOptions->pTransformerArg );
     GDALDestroyWarpOptions( psWarpOptions );
-    GDALClose( hDstDS );
-    GDALClose( hSrcDS );
+    if (hDstDS)
+		GDALClose( hDstDS );
+    if (hSrcDS)
+		GDALClose( hSrcDS );
     return false;
 }
 // #nocov end
