@@ -101,6 +101,15 @@ NumericVector CPL_inv_geotransform(NumericVector gt_r) {
 	return gt_r_inv;
 }
 
+bool equals_na(double value, double na, GDALDataType dt) {
+	if (ISNAN(value))
+		return true;
+	if (dt == GDT_Float32)
+		return (float) value == (float) na;
+	else
+		return value == na;
+}
+
 // formerly: stars/src/gdal.cpp
 NumericVector read_gdal_data(GDALDataset *poDataset, 
 			NumericVector nodatavalue, 
@@ -156,11 +165,16 @@ NumericVector read_gdal_data(GDALDataset *poDataset,
 			offset = poBand->GetOffset(NULL);
 		units[i] = poBand->GetUnitType();
 		if (! NumericVector::is_na(nodatavalue[0]) || has_offset || has_scale) {
-			for (R_xlen_t i = 0; i < Rf_xlength(vec); i++) {
-				if (vec[i] == nodatavalue[0])
-					vec[i] = NA_REAL; // #nocov
+
+
+			// for (R_xlen_t j = 0; j < Rf_xlength(vec); j++) {
+			for (R_xlen_t j = i * (((R_xlen_t) nBufXSize) * nBufYSize); // start of band i
+					j < (i + 1) * (((R_xlen_t) nBufXSize) * nBufYSize); // end of band i
+					j++) {
+				if (equals_na(vec[j], nodatavalue[0], poBand->GetRasterDataType()))
+					vec[j] = NA_REAL; // #nocov
 				else
-					vec[i] = (vec[i] * scale) + offset;
+					vec[j] = (vec[j] * scale) + offset;
 			}
 		}
 		checkUserInterrupt();
@@ -440,7 +454,7 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 		}
 
 	} else { // no create, update:
-		if ((poDstDS = (GDALDataset *) GDALOpen(fname[0], GA_Update)) == NULL)
+		if ((poDstDS = (GDALDataset *) GDALOpen(fname[0], GA_Update)) == NULL) // #nocov
 			stop("updating dataset failed"); // #nocov
 	}
 
