@@ -48,6 +48,10 @@ try(st_write(x, c("foo", "bar")))
 try(st_write(x, "foo", driver = "foo"))
 if (Sys.getenv("USER") == "travis") {
 	try(st_write(x, "/x", driver = "ESRI Shapefile"))
+	st_write(x, "xxx.gpkg")
+	st_write(x, "xxx.gpkg", quiet = FALSE) # appends to layer
+	y <- st_sf(b = 1:2, geom = st_sfc(st_point(0:1), st_multipoint(matrix(1:4,2,2))))
+	try(st_write(y, "xxx.gpkg"))
 }
 
 geom = st_sfc(st_point(0:1), st_multipoint(matrix(1:4,2,2)))
@@ -131,3 +135,11 @@ if ("GPKG" %in% st_drivers()$name) {
 	tst2 = st_read("tst__.gpkg")
 	print(identical(tst$bool, tst2$bool))
 }
+
+# spatial filter:
+nc = read_sf(system.file("gpkg/nc.gpkg", package="sf"))
+wkt = st_as_text(st_geometry(nc[1,]))
+wkt
+nc_filtered = read_sf(system.file("gpkg/nc.gpkg", package="sf"), wkt_filter = wkt)
+try(read_sf(system.file("gpkg/nc.gpkg", package="sf"), wkt_filter = "wrong"))
+nc_filtered$NAME
