@@ -9,7 +9,7 @@ test_that("st_transform works", {
   s1.tr = st_transform(s, 3857)
 
   sp = as(s, "Spatial")
-  sp.tr = spTransform(sp, CRS("+init=epsg:3857"))
+  sp.tr = spTransform(sp, CRS("+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +nadgrids=@null +no_defs")) # web mercator
   s2.tr = st_as_sfc(sp.tr)
   #attr(s1.tr, "crs")$proj4string = ""
   #attr(s2.tr, "crs")$proj4string = ""
@@ -74,6 +74,11 @@ test_that('gdal_utils work', {
   #expect_true(gdal_utils("rasterize", sd2, tf))
   expect_true(gdal_utils("translate", sd2, tf))
   expect_true(gdal_utils("vectortranslate", sd2, tf2))
+  shp = system.file("shape/nc.shp", package="sf")
+  gpkg = paste0(tempfile(), ".gpkg")
+  options = c("-f", "GPKG", "-overwrite", "-nlt", "PROMOTE_TO_MULTI", 
+  	"-oo", "ADJUST_TYPE=NO", "-doo", "FLATTEN_NESTED_ATTRIBUTES=NO")
+  expect_true(gdal_utils("vectortranslate", shp, gpkg, options = options))
   expect_warning(gdal_utils("nearblack", sd2, tf))
   # create point geom:
   points = system.file("gpkg/nc.gpkg", package="sf")
