@@ -27,6 +27,28 @@ test_that("separate and unite work", {
 	unite(CNTY_ID_NEW, c("a", "b"), sep = "") %>% inherits("sf"))
 })
 
+test_that("separate_rows work", {
+  d <- st_as_sf(data.frame(
+    x = seq_len(3),
+    y = c("a", "d,e,f", "g,h"),
+    geometry = st_sfc(st_point(c(1, 1)),
+                      st_point(c(2, 2)),
+                      st_point(c(3, 3))),
+    stringsAsFactors = FALSE))
+  expect_true(d %>%
+                separate_rows(y, convert = TRUE) %>%
+                inherits("sf"))
+  expect_identical(d %>%
+      separate_rows(y, convert = TRUE) %>%
+      st_geometry(),
+    st_sfc(st_point(c(1, 1)),
+           st_point(c(2, 2)),
+           st_point(c(2, 2)),
+           st_point(c(2, 2)),
+           st_point(c(3, 3)),
+           st_point(c(3, 3))))
+})
+
 test_that("group/ungroup works", {
  tbl = tibble(a = c(1,1,2,2), g = st_sfc(st_point(0:1), st_point(1:2), st_point(2:3), st_point(3:4)))
  d = st_sf(tbl)
@@ -39,6 +61,8 @@ test_that("sample_n etc work", {
  d = st_sf(tbl)
  sample_n(d, 2)
  sample_frac(d, .5)
+
+ # TODO: uncomment this when https://github.com/tidyverse/tidyr/pull/729 is merged
  d %>% group_by(a) %>% nest
 })
 
@@ -57,11 +81,8 @@ test_that("unnest works", {
     slice(1:2) %>%
     transmute(y = list(c("a"), c("b", "c")))
   unnest_explicit = unnest(nc, y)
-  unnest_implicit = unnest(nc)
   # The second row is duplicated because the "b" and "c" become separate rows
   expected = nc[c(1,2,2), ] %>% mutate(y = c("a", "b", "c"))
-  expected = expected[2:1]
   # Would use expect_equal, but doesn't work with geometry cols
   expect_identical(unnest_explicit, expected)
-  expect_identical(unnest_implicit, expected)
 })
